import 'package:f_r_e_hub/app_state.dart';
import 'package:f_r_e_hub/backend/api_requests/api_calls.dart';
import 'package:f_r_e_hub/flutter_flow/flutter_flow_theme.dart';
import 'package:f_r_e_hub/flutter_flow/internationalization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:provider/provider.dart';


class MessageWellComponentWidget extends StatefulWidget {
  const MessageWellComponentWidget({super.key});

  @override
  State<MessageWellComponentWidget> createState() =>
      _MessageWellComponentWidgetState();
}

class _MessageWellComponentWidgetState extends State<MessageWellComponentWidget> {
  late ScrollController _listViewController;
  bool _allowScrollInSingleChildScrollView = true;
  
  VoidCallback get _scrollListener => MessageWellNotifier()._scrollListener;
  final dropdownValueNotifier = ValueNotifier<String>('All'); // Using ValueNotifier for state management


  @override
  void initState() {
    super.initState();
    _listViewController = ScrollController();
    _listViewController.addListener(_scrollListener); // Adicione esta linha
    // WidgetsBinding.instance?.addPostFrameCallback((_) {
    //   context.read<MessageWellNotifier>().fetchMessages();
    // });
  }

  @override
  void dispose() {
    _listViewController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final state = context.watch<MessageWellNotifier>();
    final messages = state.getMessages();
    final theme = FlutterFlowTheme.of(context);
    return Align(
      alignment: Alignment.center,
      child: Padding(
        padding: const EdgeInsets.symmetric(vertical: 40.0),
        child: SizedBox(
          height: MediaQuery.of(context).size.height * 0.8,
          child: Column(
            children: [
              _buildHandleMessageWell(context, theme),
              _buildMenuMessageWell(context, theme, dropdownValueNotifier),
              Expanded(
                child: NotificationListener<ScrollNotification>(
                  onNotification: _buildScrollMessageListManager,
                  child: ListView.builder(
                    controller: _listViewController,
                    itemCount: messages.length,
                    shrinkWrap: true,
                    physics: const AlwaysScrollableScrollPhysics(),
                    itemBuilder: (context, index) {
                      var message = messages[index];
                      return _buildMessageItem(context, message, index);
                    },
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }


bool _buildScrollMessageListManager(ScrollNotification notification) {
  final notifier = context.read<MessageWellNotifier>();
  if (notification is ScrollUpdateNotification) {
    debugPrint('Scrolling ...');
    if (_listViewController.offset <= 0 && notification.scrollDelta! < 0) {
      debugPrint('Scrolling up ...');
      setState(() {
        _allowScrollInSingleChildScrollView = true;
      });
    } else {
      debugPrint('Scrolling down ...');
      if (_listViewController.position.extentAfter == 0) {
        debugPrint('Reached end of list. Incrementing pageNumber...');
        notifier.incrementPageNumber();
      }
      setState(() {
        _allowScrollInSingleChildScrollView = false;
        // MessageWellNotifier().fetchMessages();
      });
    }
  }
  return true;
}

Widget _buildMenuMessageWell(BuildContext context, FlutterFlowTheme theme, ValueNotifier<String> dropdownValueNotifier) {

  final locations = FFLocalizations.of(context);
  late String all = locations.getVariableText(enText: 'All', ptText: 'All');
  late String personal = locations.getVariableText(enText: 'Personal', ptText: 'Pessoal');
  late String global = locations.getVariableText(enText: 'Global', ptText: 'Global');

  return SizedBox(
    key: UniqueKey(),
    width: 200,
    height: 40,
    child: ValueListenableBuilder<String>(
      valueListenable: dropdownValueNotifier,
      builder: (context, value, child) {
        return DropdownButtonFormField<String>(
          value: value,
          decoration: InputDecoration(
            contentPadding: EdgeInsets.symmetric(horizontal: 10.0),
            border: OutlineInputBorder(
              borderSide: BorderSide(
                color: theme.primary, // Assuming FlutterFlowTheme.of(context).primary is the correct usage
                width: 2, // Adjusted width to a reasonable value
              ),
              borderRadius: BorderRadius.circular(10),
            ),
            filled: true,
            fillColor: theme.primary,
          ),
          onChanged: (String? newValue) {
            if (newValue != null) {
              // logger.log('DropDown value: $newValue'); // Assuming logger is correctly initialized elsewhere
              dropdownValueNotifier.value = newValue; // Update the ValueNotifier value
            }
          },
          items: [
            DropdownMenuItem<String>(value: all, child: Text(all)),
            DropdownMenuItem<String>(value: personal, child: Text(personal)),
            DropdownMenuItem<String>(value: global, child: Text(global)),
          ],
          style: theme.labelMedium.override(
            fontFamily: theme.labelMediumFamily,
            color: theme.primaryText,
            useGoogleFonts: GoogleFonts.asMap().containsKey(theme.labelMediumFamily),
          ),
        );
      },
    ),
  );
}

  Text _buildHandleMessageWell(BuildContext context, FlutterFlowTheme theme) {
    return Text(
      FFLocalizations.of(context).getText('8fworxmb'), // Mural de Mensagens
      style: theme.bodyMedium.override(
        fontFamily: 'Nunito Sans',
        letterSpacing: 0.0,
        useGoogleFonts: GoogleFonts.asMap().containsKey('Nunito Sans'),
      ),
    );
  }

  Widget _buildMessageItem(BuildContext context, dynamic message, int index) {
  final theme = Theme.of(context);
  String formatMessageOrigin(String messageOrigin) {
    final words = messageOrigin.split(' ');
    final formattedWords = words.map((word) {
      final firstLetter = word.substring(0, 1).toUpperCase();
      final remainingLetters = word.substring(1).toLowerCase();
      return '$firstLetter$remainingLetters';
    });
    return formattedWords.join(' ');
  }
  return GestureDetector(
    onTap: () => print('Message tapped ...\nmessage[$index]: $message'),
    child: Padding(
      padding: const EdgeInsets.fromLTRB(20, 5, 20, 5),
      child: Container(
        width: MediaQuery.of(context).size.width * 0.9,
        height: 127.0,
        decoration: const BoxDecoration(
          borderRadius: BorderRadius.all(Radius.circular(10)),
        ),
        child: Row(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            Padding(
              padding: const EdgeInsets.fromLTRB(0, 0, 0, 0),
              child: Container(
                width: 64.0,
                height: 64.0,
                decoration: const BoxDecoration(shape: BoxShape.circle),
              ),
            ),
            Expanded(
              child: Column(
                mainAxisSize: MainAxisSize.max,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  const SizedBox(height: 8.0),
                  Center(
                    child: Text(
                        '~ ${formatMessageOrigin(message['MSG_ORIGEM_DESC'].toString())}',
                        style: theme.textTheme.headlineMedium!.copyWith(
                                fontFamily: 'Nunito Sans',
                                color: FlutterFlowTheme.of(context).primary,
                                fontSize: 14.0,
                                fontWeight: FontWeight.bold,
                              ),
                        ),
                      ),
                  const SizedBox(height: 8.0),
                  Expanded(
                    child: SingleChildScrollView(
                      scrollDirection: Axis.vertical,
                      child: Text(
                        formatMessageOrigin(message['MSG_TEXTO'].toString()),
                        style: theme.textTheme.bodyMedium!.copyWith(
                          fontFamily: 'Nunito Sans',
                          color: FlutterFlowTheme.of(context).primaryText,
                          fontSize: 14.0,
                        ),
                        softWrap: true,
                        maxLines: 10,
                      ),
                    ),
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    ),
  );
}
}

class MessageWellState {
  final List<dynamic> messages;
  int pageNumber;
  final bool allowScrollInSingleChildScrollView;
  final ScrollController listViewController;

  MessageWellState({
    required this.messages,
    this.pageNumber = 1,
    required this.allowScrollInSingleChildScrollView,
    required this.listViewController,
  });

  MessageWellState copyWith({
    List<dynamic>? messages,
    int? pageNumber,
    bool? allowScrollInSingleChildScrollView,
    ScrollController? listViewController,
  }) {
    return MessageWellState(
      messages: messages ?? this.messages,
      pageNumber: pageNumber ?? this.pageNumber,
      allowScrollInSingleChildScrollView: allowScrollInSingleChildScrollView ?? this.allowScrollInSingleChildScrollView,
      listViewController: listViewController ?? this.listViewController,
    );
  }
}

class MessageWellNotifier extends StateNotifier<MessageWellState> {
  var totalPageNumber = 1;

  MessageWellNotifier()
      : super(MessageWellState(messages: [], allowScrollInSingleChildScrollView: true, listViewController: ScrollController())) {
    state.listViewController.addListener(_scrollListener);
    fetchMessages();
  }

  void fetchMessages() async {
    if (state.pageNumber <= totalPageNumber) {
      var apiCall = GetMessagesCall();
      var response = await apiCall.call(
        devUUID: FFAppState().devUUID.toString(),
        userUUID: FFAppState().userUUID.toString(),
        cliID: FFAppState().cliUUID.toString(),
        atividade: 'getMensagens',
        pageSize: '10',
        pageNumber: state.pageNumber.toString(),
        tipoDestino: 'A',
      );
      if (response.statusCode == 200) {
        var messagesData = response.jsonBody['mensagens'];
        debugPrint('inputs: ${FFAppState().devUUID}, ${FFAppState().userUUID}, ${FFAppState().cliUUID}');
        debugPrint('response: ${response.jsonBody}');
        debugPrint('response[mensagens]: $messagesData');
        debugPrint('response[mensagens][0][MSG_TEXTO]: ${messagesData[0]['MSG_TEXTO']}');
        var newMessages = [...state.messages, ...messagesData];
        var rExp = RegExp(r'\d+').allMatches(newMessages.toString()).map((e) => e.group(0)).toList();
        debugPrint('rExp: $rExp');
        debugPrint('newMessages: $newMessages');
        state = state.copyWith(messages: newMessages);
        totalPageNumber = int.parse(response.jsonBody['total_pages']);
        debugPrint('totalPageNumber: $totalPageNumber');
      } else {
        debugPrint('Error fetching messages: ${response.statusCode}');
      }
    } else {
      debugPrint('No more messages to fetch ...');
    }
  }

  List<dynamic> getMessages() {
    return state.messages;
  }

  void _scrollListener() {
    if (!state.allowScrollInSingleChildScrollView) {
      if (state.pageNumber <= totalPageNumber) {
        state = state.copyWith(pageNumber: state.pageNumber + 1);
        debugPrint('state.pageNumber: ${state.pageNumber}');
        fetchMessages();
      }
    }
  }

  @override
  void dispose() {
    state.listViewController.removeListener(_scrollListener);
    super.dispose();
  }
  
  void incrementPageNumber() {
    if (state.pageNumber <= totalPageNumber) {
      state = state.copyWith(pageNumber: state.pageNumber + 1);
      debugPrint('state.pageNumber: ${state.pageNumber}');
    }
  }
}


mixin Logger {
  static void log(String message) {
    debugPrint('Logger: $message');
  }
}

final messageWellProvider = StateNotifierProvider<MessageWellNotifier, MessageWellState>((ref) {
  return MessageWellNotifier();
});
